#include "OmHeaders.h"

// Hibernate a node to an OpenMath output device. The writing follows a Standard OpenMath protocol, so that
// the node, its attributes and its adopted subtree are all stored into the device.
void OmNode::hibernate(OmOutputDevice & output_in, const OmNode * node_in)
{
  OmException::checkNullPointer(node_in);

  for (comments_t::const_iterator it=node_in->comments_.begin(); it!=node_in->comments_.end(); ++it)
    {
      ((*it))->write(output_in);
    }

  if (node_in->attributes_.empty())
    {
      node_in->writeImp(output_in);
    }
  else
    {
      output_in.writeAttribute();
      output_in.writeAttributeParameter();
      for (attributes_t::const_iterator it=node_in->attributes_.begin(); it!=node_in->attributes_.end(); ++it)
	{
	  hibernate(output_in, (*it).first);
	  hibernate(output_in, (*it).second);
	}
      output_in.writeEndAttributeParameter();
      node_in->writeImp(output_in);
      output_in.writeEndAttribute();
    }
}

// Resurrect the node from an OpenMath input device. The reading follows a Standard OpenMath protocol, so that
// the node, its attributes and its adopted subtree are all restored from the device
OmNode * OmNode::resurrect(OmInputDevice & input_in)
{
  OmNode * result = 0;
  OmType type;

  input_in.readType(type);

  // Attributed object => non terminal parsing (recursive)
  if (type==OmAttributeType)
    {
      OmCommentCollector cc;
      input_in.readAttribute();
      cc.collect(input_in);
      input_in.readAttributeParameter();
      cc.collect(input_in);
      input_in.readType(type);
      vector<attributes_t::value_type> attributes;
      while (type!=OmEndAttributeParameterType)
	{
	  OmSymbolNode * symbol = (OmSymbolNode *)cc.uncollect(resurrect(input_in));
	  OmException::checkMalformedNode(symbol->type()!=OmSymbolType, "An OmSymbolNode was expected in attribution");
	  cc.collect(input_in);
	  OmNode * node = cc.uncollect(resurrect(input_in));
	  attributes.push_back(attributes_t::value_type(symbol, node));
	  cc.collect(input_in);
	  input_in.readType(type);
	}
      input_in.readEndAttributeParameter();
      cc.collect(input_in);
      result = cc.uncollect(resurrect(input_in));
      cc.collect(input_in);
      input_in.readEndAttribute();
      result->attributes_.insert(result->attributes_.begin(), attributes.begin(), attributes.end());
    }
  // Not attributed object => terminal parsing
  else
    {
      result = create(type);
      result->readImp(input_in);
    }

  assert(result!=0);
  return result;
}

// Dynamically create a new node given its type. Breaking the OCP makes any descendant dependent from this unit.
// Note that all nodes have to be default constructible.
OmNode * OmNode::create(OmType type_in)
{
  switch (type_in)
    {
    case OmIntegerType      : return new OmIntegerNode;
    case OmBigIntegerType   : return new OmBigIntegerNode;
    case OmFloatType        : return new OmFloatNode;
    case OmByteArrayType    : return new OmByteArrayNode;
    case OmStringType       : return new OmStringNode;
    case OmWStringType      : return new OmWStringNode;
    case OmSymbolType       : return new OmSymbolNode;
    case OmVariableType     : return new OmVariableNode;
    case OmPInstructionType : return new OmPInstructionNode;
    case OmApplicationType  : return new OmApplicationNode;
    case OmBindingType      : return new OmBindingNode;
    case OmErrorType        : return new OmErrorNode;
    case OmObjectType       : return new OmObjectNode;
    default                 : throw OmException(OmException::Other, "Illegal OmType or does not correspond to a creatable object.");
    }
}
